@@ -2,13 +2,17 @@
 module WaiAppStaticTest (spec) where
 
 import Network.Wai.Application.Static
+import WaiAppStatic.Types
 
 import Test.Hspec
 import qualified Data.ByteString.Char8 as S8
 -- import qualified Data.ByteString.Lazy.Char8 as L8
 import System.PosixCompat.Files (getFileStatus, modificationTime)
+import System.FilePath
+import System.IO.Temp
 
 import Network.HTTP.Date
+import Network.HTTP.Types (status500)
 {-import System.Locale (defaultTimeLocale)-}
 {-import Data.Time.Format (formatTime)-}
 
@@ -24,7 +28,10 @@ defRequest = defaultRequest
 spec :: Spec
 spec = do
   let webApp = flip runSession $ staticApp $ defaultWebAppSettings "test"
-  let fileServerApp = flip runSession $ staticApp $ defaultFileServerSettings "test"
+  let fileServerAppWithSettings settings = flip runSession $ staticApp settings
+  let fileServerApp = fileServerAppWithSettings (defaultFileServerSettings "test")
+        { ssAddTrailingSlash = True
+        }
 
   let etag = "1B2M2Y8AsgTpgAmY7PhCfg=="
   let file = "a/b"
@@ -123,3 +130,44 @@ spec = do
       assertStatus 304 req
       assertNoHeader "Cache-Control" req
 
+    context "301 redirect to add a trailing slash on directories if missing" $ do
+      it "works at the root" $ fileServerApp $ do
+        req <- request (setRawPathInfo defRequest "/a")
+        assertStatus 301 req
+        assertHeader "Location" "/a/" req
+
+      let urlMapApp = flip runSession $ \req send ->
+            case pathInfo req of
+                "subPath":rest ->
+                    let req' = req { pathInfo = rest }
+                     in (staticApp (defaultFileServerSettings "test")
+                            { ssAddTrailingSlash = True
+                            }) req' send
+                _ -> send $ responseLBS status500 []
+                    "urlMapApp: only works at subPath"
+      it "works with subpath at the root of the file server" $ urlMapApp $ do
+        req <- request (setRawPathInfo defRequest "/subPath")
+        assertStatus 301 req
+        assertHeader "Location" "/subPath/" req
+
+    context "with defaultWebAppSettings" $ do
+      it "ssIndices works" $ do
+        withSystemTempDirectory "wai-app-static-test" $ \ dir -> do
+          writeFile (dir </> "index.html") "foo"
+          let testSettings = (defaultWebAppSettings dir) {
+                ssIndices = [unsafeToPiece "index.html"]
+              }
+          fileServerAppWithSettings testSettings $ do
+            resp <- request (setRawPathInfo defRequest "/")
+            assertStatus 200 resp
+            assertBody "foo" resp
+
+    context "with defaultFileServerSettings" $ do
+      it "prefers ssIndices over ssListing" $ do
+        withSystemTempDirectory "wai-app-static-test" $ \ dir -> do
+          writeFile (dir </> "index.html") "foo"
+          let testSettings = defaultFileServerSettings dir
+          fileServerAppWithSettings testSettings $ do
+            resp <- request (setRawPathInfo defRequest "/")
+            assertStatus 200 resp
+            assertBody "foo" resp
